package com.fjhx.oa.flow;

import com.alibaba.fastjson.JSONObject;
import com.fjhx.common.enums.CodingRuleEnum;
import com.fjhx.common.enums.FlowStatusEnum1;
import com.fjhx.common.service.coding.CodingRuleService;
import com.fjhx.flow.core.FlowDelegate;
import com.fjhx.flow.enums.FlowStatusEnum;
import com.fjhx.oa.entity.personal.dto.PersonalInvoiceDto;
import com.fjhx.oa.entity.personal.po.PersonalInvoice;
import com.fjhx.oa.service.personal.PersonalInvoiceService;
import com.ruoyi.common.core.domain.BasePo;
import com.ruoyi.common.utils.SecurityUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Date;

@Component
public class PersonalInvoiceFlow extends FlowDelegate {

    @Autowired
    private PersonalInvoiceService personalInvoiceService;
    @Autowired
    private CodingRuleService codingRuleService;

    @Override
    public String getFlowKey() {
        return "personal_invoice_flow";
    }

    @Override
    public Long start(Long flowId, JSONObject submitData) {
        PersonalInvoiceDto dto = submitData.toJavaObject(PersonalInvoiceDto.class);
        dto.setFlowId(flowId);

        dto.setCode(codingRuleService.createCode(CodingRuleEnum.PERSONAL_INVOICE.getKey(), null));
        dto.setStatus(FlowStatusEnum1.UNDER_REVIEW.getKey());

        personalInvoiceService.addOrEdit(dto);

        return dto.getId();
    }

    @Override
    public void end(Long flowId, Long businessId, JSONObject submitData) {
        personalInvoiceService.update(q -> q
                .eq(PersonalInvoice::getId, businessId)
                .set(PersonalInvoice::getStatus, FlowStatusEnum1.PASS.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void relaunch(Long flowId, Long businessId, FlowStatusEnum flowStatus, JSONObject submitData) {
        start(flowId, submitData);
    }

    @Override
    public void reject(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        personalInvoiceService.update(q -> q
                .eq(PersonalInvoice::getId, businessId)
                .set(PersonalInvoice::getStatus, FlowStatusEnum1.REJECT.getKey())
                .set(BasePo::getUpdateTime, new Date())
                .set(BasePo::getUpdateUser, SecurityUtils.getUserId())
        );
    }

    @Override
    public void cancellation(Long flowId, Long businessId, FlowStatusEnum flowStatus) {
        personalInvoiceService.cancellation(businessId);
    }
}
